# Autogenerated. Do not modify.
#
# Profile: 21.60
from fit_tool.base_type import BaseType
from fit_tool.field import Field


class TimestampField(Field):
    ID = 253

    def __init__(self, size: int = 0, growable: bool = True):
        super().__init__(name='timestamp', field_id=self.ID, base_type=BaseType.UINT32, offset=-631065600000,
                         scale=0.001,
                         size=size,
                         units='ms', type_name='date_time', growable=growable, sub_fields=[])


class MessageIndexField(Field):
    ID = 254

    def __init__(self, size: int = 0, growable: bool = True):
        super().__init__(name='message_index', field_id=self.ID, base_type=BaseType.UINT16, size=size, units='ms',
                         type_name='date_time', growable=growable, sub_fields=[])
